@@ -43,11 +43,6 @@ export function helper(babel, options) {
   const JSX_ANNOTATION_REGEX = /\*?\s*@jsx\s+([^\s]+)/;
   const JSX_FRAG_ANNOTATION_REGEX = /\*?\s*@jsxFrag\s+([^\s]+)/;
 
-  // This is the number of possible import names
-  // development: jsxDEV, Fragment, createElement
-  // production: jsx, jsxs, Fragment, createElement
-  const IMPORT_NAME_SIZE = options.development ? 3 : 4;
-
   const {
     importSource: IMPORT_SOURCE_DEFAULT = DEFAULT.importSource,
     runtime: RUNTIME_DEFAULT = DEFAULT.runtime,
@@ -138,136 +133,123 @@ You can set \`throwIfNamespace: false\` to bypass this warning.`,
 
     Program: {
       enter(path, state) {
-        if (hasJSX(path)) {
-          const { file } = state;
-          let runtime = RUNTIME_DEFAULT;
+        const { file } = state;
+        let runtime = RUNTIME_DEFAULT;
 
-          // For jsx mode
-          let source = IMPORT_SOURCE_DEFAULT;
-          let sourceSet = !!options.importSource;
+        // For jsx mode
+        let source = IMPORT_SOURCE_DEFAULT;
+        let sourceSet = !!options.importSource;
 
-          // For createElement mode
-          let pragma = PRAGMA_DEFAULT;
-          let pragmaFrag = PRAGMA_FRAG_DEFAULT;
-          let pragmaSet = !!options.pragma;
-          let pragmaFragSet = !!options.pragmaFrag;
+        // For createElement mode
+        let pragma = PRAGMA_DEFAULT;
+        let pragmaFrag = PRAGMA_FRAG_DEFAULT;
+        let pragmaSet = !!options.pragma;
+        let pragmaFragSet = !!options.pragmaFrag;
 
-          if (file.ast.comments) {
-            for (const comment of (file.ast.comments: Array<Object>)) {
-              const sourceMatches = JSX_SOURCE_ANNOTATION_REGEX.exec(
-                comment.value,
-              );
-              if (sourceMatches) {
-                source = sourceMatches[1];
-                sourceSet = true;
-              }
+        if (file.ast.comments) {
+          for (const comment of (file.ast.comments: Array<Object>)) {
+            const sourceMatches = JSX_SOURCE_ANNOTATION_REGEX.exec(
+              comment.value,
+            );
+            if (sourceMatches) {
+              source = sourceMatches[1];
+              sourceSet = true;
+            }
 
-              const runtimeMatches = JSX_RUNTIME_ANNOTATION_REGEX.exec(
-                comment.value,
-              );
-              if (runtimeMatches) {
-                runtime = runtimeMatches[1];
-              }
+            const runtimeMatches = JSX_RUNTIME_ANNOTATION_REGEX.exec(
+              comment.value,
+            );
+            if (runtimeMatches) {
+              runtime = runtimeMatches[1];
+            }
 
-              const jsxMatches = JSX_ANNOTATION_REGEX.exec(comment.value);
-              if (jsxMatches) {
-                pragma = jsxMatches[1];
-                pragmaSet = true;
-              }
-              const jsxFragMatches = JSX_FRAG_ANNOTATION_REGEX.exec(
-                comment.value,
-              );
-              if (jsxFragMatches) {
-                pragmaFrag = jsxFragMatches[1];
-                pragmaFragSet = true;
-              }
+            const jsxMatches = JSX_ANNOTATION_REGEX.exec(comment.value);
+            if (jsxMatches) {
+              pragma = jsxMatches[1];
+              pragmaSet = true;
+            }
+            const jsxFragMatches = JSX_FRAG_ANNOTATION_REGEX.exec(
+              comment.value,
+            );
+            if (jsxFragMatches) {
+              pragmaFrag = jsxFragMatches[1];
+              pragmaFragSet = true;
             }
           }
+        }
 
-          state.set("@babel/plugin-react-jsx/runtime", runtime);
-          if (runtime === "classic") {
-            if (sourceSet) {
-              throw path.buildCodeFrameError(
-                `importSource cannot be set when runtime is classic.`,
-              );
-            }
-            state.set(
-              "@babel/plugin-react-jsx/createElementIdentifier",
-              createIdentifierParser(pragma),
-            );
-            state.set(
-              "@babel/plugin-react-jsx/jsxFragIdentifier",
-              createIdentifierParser(pragmaFrag),
-            );
-            state.set("@babel/plugin-react-jsx/usedFragment", false);
-            state.set(
-              "@babel/plugin-react-jsx/pragmaSet",
-              pragma !== DEFAULT.pragma,
-            );
-            state.set(
-              "@babel/plugin-react-jsx/pragmaFragSet",
-              pragmaFrag !== DEFAULT.pragmaFrag,
-            );
-          } else if (runtime === "automatic") {
-            if (pragmaSet || pragmaFragSet) {
-              throw path.buildCodeFrameError(
-                `pragma and pragmaFrag cannot be set when runtime is automatic.`,
-              );
-            }
-
-            const importName = addAutoImports(path, {
-              ...state.opts,
-              source,
-            });
-
-            state.set(
-              "@babel/plugin-react-jsx/jsxIdentifier",
-              createIdentifierParser(
-                createIdentifierName(
-                  path,
-                  options.development ? "jsxDEV" : "jsx",
-                  importName,
-                ),
-              ),
-            );
-            state.set(
-              "@babel/plugin-react-jsx/jsxStaticIdentifier",
-              createIdentifierParser(
-                createIdentifierName(
-                  path,
-                  options.development ? "jsxDEV" : "jsxs",
-                  importName,
-                ),
-              ),
-            );
-
-            state.set(
-              "@babel/plugin-react-jsx/createElementIdentifier",
-              createIdentifierParser(
-                createIdentifierName(path, "createElement", importName),
-              ),
-            );
-
-            state.set(
-              "@babel/plugin-react-jsx/jsxFragIdentifier",
-              createIdentifierParser(
-                createIdentifierName(path, "Fragment", importName),
-              ),
-            );
-
-            state.set(
-              "@babel/plugin-react-jsx/importSourceSet",
-              source !== DEFAULT.importSource,
-            );
-          } else {
+        state.set("@babel/plugin-react-jsx/runtime", runtime);
+        if (runtime === "classic") {
+          if (sourceSet) {
             throw path.buildCodeFrameError(
-              `Runtime must be either "classic" or "automatic".`,
+              `importSource cannot be set when runtime is classic.`,
+            );
+          }
+          state.set(
+            "@babel/plugin-react-jsx/createElementIdentifier",
+            createIdentifierParser(pragma),
+          );
+          state.set(
+            "@babel/plugin-react-jsx/jsxFragIdentifier",
+            createIdentifierParser(pragmaFrag),
+          );
+          state.set("@babel/plugin-react-jsx/usedFragment", false);
+          state.set(
+            "@babel/plugin-react-jsx/pragmaSet",
+            pragma !== DEFAULT.pragma,
+          );
+          state.set(
+            "@babel/plugin-react-jsx/pragmaFragSet",
+            pragmaFrag !== DEFAULT.pragmaFrag,
+          );
+        } else if (runtime === "automatic") {
+          if (pragmaSet || pragmaFragSet) {
+            throw path.buildCodeFrameError(
+              `pragma and pragmaFrag cannot be set when runtime is automatic.`,
             );
           }
 
-          if (options.development) {
-            path.traverse(injectMetaPropertiesVisitor, state);
-          }
+          state.set(
+            "@babel/plugin-react-jsx/jsxIdentifier",
+            createImportLazily(
+              state,
+              path,
+              options.development ? "jsxDEV" : "jsx",
+              source,
+            ),
+          );
+          state.set(
+            "@babel/plugin-react-jsx/jsxStaticIdentifier",
+            createImportLazily(
+              state,
+              path,
+              options.development ? "jsxDEV" : "jsxs",
+              source,
+            ),
+          );
+
+          state.set(
+            "@babel/plugin-react-jsx/createElementIdentifier",
+            createImportLazily(state, path, "createElement", source),
+          );
+
+          state.set(
+            "@babel/plugin-react-jsx/jsxFragIdentifier",
+            createImportLazily(state, path, "Fragment", source),
+          );
+
+          state.set(
+            "@babel/plugin-react-jsx/importSourceSet",
+            source !== DEFAULT.importSource,
+          );
+        } else {
+          throw path.buildCodeFrameError(
+            `Runtime must be either "classic" or "automatic".`,
+          );
+        }
+
+        if (options.development) {
+          path.traverse(injectMetaPropertiesVisitor, state);
         }
       },
 
@@ -315,56 +297,6 @@ You can set \`throwIfNamespace: false\` to bypass this warning.`,
     return false;
   }
 
-  function createIdentifierName(path, name, importName) {
-    if (isModule(path)) {
-      const identifierName = `${importName[name]}`;
-      return identifierName;
-    } else {
-      return `${importName[name]}.${name}`;
-    }
-  }
-
-  function getImportNames(parentPath) {
-    const imports = new Set();
-
-    parentPath.traverse({
-      "JSXElement|JSXFragment"(path) {
-        if (path.type === "JSXFragment") imports.add("Fragment");
-        const openingPath = path.get("openingElement");
-
-        const validChildren = t.react.buildChildren(openingPath.parent);
-        let importName;
-        if (path.type === "JSXElement" && shouldUseCreateElement(path)) {
-          importName = "createElement";
-        } else if (options.development) {
-          importName = "jsxDEV";
-        } else if (validChildren.length > 1) {
-          importName = "jsxs";
-        } else {
-          importName = "jsx";
-        }
-        imports.add(importName);
-
-        if (imports.size === IMPORT_NAME_SIZE) {
-          path.stop();
-        }
-      },
-    });
-    return imports;
-  }
-
-  function hasJSX(parentPath) {
-    let fileHasJSX = false;
-    parentPath.traverse({
-      "JSXElement|JSXFragment"(path) {
-        fileHasJSX = true;
-        path.stop();
-      },
-    });
-
-    return fileHasJSX;
-  }
-
   function getSource(source, importName) {
     switch (importName) {
       case "Fragment":
@@ -381,47 +313,40 @@ You can set \`throwIfNamespace: false\` to bypass this warning.`,
     }
   }
 
-  function addAutoImports(path, state) {
-    const imports = getImportNames(path, state);
-    if (isModule(path)) {
-      // import {jsx} from "react";
-      // import {createElement} from "react";
-      const importMap = {};
+  function createImportLazily(pass, path, importName, source) {
+    return () => {
+      const actualSource = getSource(source, importName);
+      if (isModule(path)) {
+        let reference = pass.get(
+          `@babel/plugin-react-jsx/imports/${importName}`,
+        );
+        if (reference) return t.cloneNode(reference);
 
-      imports.forEach(importName => {
-        if (!importMap[importName]) {
-          importMap[importName] = addNamed(
-            path,
-            importName,
-            getSource(state.source, importName),
-            {
-              importedInterop: "uncompiled",
-              ensureLiveReference: true,
-            },
-          ).name;
+        reference = addNamed(path, importName, actualSource, {
+          importedInterop: "uncompiled",
+        });
+        pass.set(`@babel/plugin-react-jsx/imports/${importName}`, reference);
+
+        return reference;
+      } else {
+        let reference = pass.get(
+          `@babel/plugin-react-jsx/requires/${actualSource}`,
+        );
+        if (reference) {
+          reference = t.cloneNode(reference);
+        } else {
+          reference = addNamespace(path, actualSource, {
+            importedInterop: "uncompiled",
+          });
+          pass.set(
+            `@babel/plugin-react-jsx/requires/${actualSource}`,
+            reference,
+          );
         }
-      });
 
-      return importMap;
-    } else {
-      const importMap = {};
-      const sourceMap = {};
-      imports.forEach(importName => {
-        const source = getSource(state.source, importName);
-        if (!importMap[importName]) {
-          if (!sourceMap[source]) {
-            // var _react = require("react")
-            sourceMap[source] = addNamespace(path, source, {
-              importedInterop: "uncompiled",
-              ensureLiveReference: true,
-            }).name;
-          }
-
-          importMap[importName] = sourceMap[source];
-        }
-      });
-      return importMap;
-    }
+        return t.memberExpression(reference, t.identifier(importName));
+      }
+    };
   }
 
   function createIdentifierParser(id) {
@@ -851,7 +776,6 @@ You can set \`throwIfNamespace: false\` to bypass this warning.`,
         if (name === "__source" || name === "__self") {
           if (found[name]) throw sourceSelfError(path, name);
           found[name] = true;
-          if (!options.development) continue;
         }
 
         props.push(convertAttribute(attr));
@@ -862,20 +786,8 @@ You can set \`throwIfNamespace: false\` to bypass this warning.`,
 
     let props = [];
     const objs = [];
-    const found = Object.create(null);
 
     for (const attr of attribs) {
-      const name =
-        t.isJSXAttribute(attr) &&
-        t.isJSXIdentifier(attr.name) &&
-        attr.name.name;
-
-      if (name === "__source" || name === "__self") {
-        if (found[name]) throw sourceSelfError(path, name);
-        found[name] = true;
-        if (!options.development) continue;
-      }
-
       if (useSpread || !t.isJSXSpreadAttribute(attr)) {
         props.push(convertAttribute(attr));
       } else {
